"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var query_exports = {};
__export(query_exports, {
  Db2Query: () => Db2Query
});
module.exports = __toCommonJS(query_exports);
var import_node_assert = __toESM(require("node:assert"));
var import_query = require("../abstract/query");
var import_logger = require("../../utils/logger");
const sequelizeErrors = require("../../errors");
const _ = require("lodash");
const debug = import_logger.logger.debugContext("sql:db2");
class Db2Query extends import_query.AbstractQuery {
  getInsertIdField() {
    return "id";
  }
  getSQLTypeFromJsType(value) {
    if (Buffer.isBuffer(value)) {
      return { ParamType: "INPUT", DataType: "BLOB", Data: value };
    }
    if (typeof value === "bigint") {
      return value.toString();
    }
    return value;
  }
  async _run(connection, sql, parameters) {
    (0, import_node_assert.default)(typeof sql === "string", `sql parameter must be a string`);
    this.sql = sql;
    const complete = this._logQuery(sql, debug, parameters);
    if (this.sql.startsWith("BEGIN TRANSACTION")) {
      try {
        await connection.beginTransaction();
      } catch (error) {
        throw this.formatError(error);
      }
      return this.formatResults();
    }
    if (this.sql.startsWith("COMMIT TRANSACTION")) {
      try {
        await connection.commitTransaction();
      } catch (error) {
        throw this.formatError(error);
      }
      return this.formatResults();
    }
    if (this.sql.startsWith("ROLLBACK TRANSACTION")) {
      try {
        await connection.rollbackTransaction();
      } catch (error) {
        throw this.formatError(error);
      }
      return this.formatResults();
    }
    if (this.sql.startsWith("SAVE TRANSACTION")) {
      try {
        await connection.commitTransaction();
        await connection.beginTransaction();
      } catch (error) {
        throw this.formatError(error);
      }
      return this.formatResults();
    }
    const params = [];
    if (parameters) {
      _.forOwn(parameters, (value, key) => {
        const param = this.getSQLTypeFromJsType(value, key);
        params.push(param);
      });
    }
    const SQL = this.sql.toUpperCase();
    let newSql = this.sql;
    if ((this.isSelectQuery() || SQL.startsWith("SELECT ")) && !SQL.includes(" FROM ", 8)) {
      if (this.sql.charAt(this.sql.length - 1) === ";") {
        newSql = this.sql.slice(0, -1);
      }
      newSql += " FROM SYSIBM.SYSDUMMY1;";
    }
    let stmt;
    try {
      stmt = await connection.prepare(newSql);
    } catch (error) {
      throw this.formatError(error);
    }
    let res;
    try {
      res = await this.#execute(stmt, params);
    } catch (error) {
      if (error.message) {
        error = this.filterSQLError(error, this.sql, connection);
        if (error === null) {
          stmt.closeSync();
          return this.formatResults([], 0);
        }
      }
      error.sql = sql;
      stmt.closeSync();
      throw this.formatError(error, connection, parameters);
    }
    const { result, outparams } = res;
    complete();
    if (outparams && this.options.bindParameterOrder && this.options._unsafe_db2Outparams) {
      for (let i = 0; i < this.options.bindParameterOrder.length; i++) {
        const paramName = this.options.bindParameterOrder[i];
        const paramValue = outparams[i];
        this.options._unsafe_db2Outparams.set(paramName, paramValue);
      }
    }
    let data = [];
    let metadata = [];
    let affectedRows = 0;
    if (typeof result === "object") {
      if (this.sql.startsWith("DELETE FROM ")) {
        affectedRows = result.getAffectedRowsSync();
      } else {
        data = result.fetchAllSync();
        metadata = result.getColumnMetadataSync();
      }
      result.closeSync();
    }
    stmt.closeSync();
    const datalen = data.length;
    if (datalen > 0) {
      const coltypes = {};
      for (const metadatum of metadata) {
        coltypes[metadatum.SQL_DESC_NAME] = metadatum.SQL_DESC_TYPE_NAME;
      }
      for (let i = 0; i < datalen; i++) {
        for (const column in data[i]) {
          const value = data[i][column];
          if (value === null) {
            continue;
          }
          const parse = this.sequelize.dialect.getParserForDatabaseDataType(coltypes[column]);
          if (parse) {
            data[i][column] = parse(value);
          }
        }
      }
      if (outparams && outparams.length > 0) {
        data.unshift(outparams);
      }
      return this.formatResults(data, datalen, metadata, connection);
    }
    return this.formatResults(data, affectedRows);
  }
  async run(sql, parameters) {
    return await this._run(this.connection, sql, parameters);
  }
  #execute(stmt, params) {
    return new Promise((resolve, reject) => {
      stmt.execute(params, (err, result, outparams) => {
        if (err) {
          reject(err);
        } else {
          resolve({ result, outparams });
        }
      });
    });
  }
  filterSQLError(err, sql, connection) {
    if (err.message.search("SQL0605W") !== -1) {
      return null;
    }
    return err;
  }
  /**
   * High level function that handles the results of a query execution.
   *
   *
   * Example:
   *  query.formatResults([
   *    {
   *      id: 1,              // this is from the main table
   *      attr2: 'snafu',     // this is from the main table
   *      Tasks.id: 1,        // this is from the associated table
   *      Tasks.title: 'task' // this is from the associated table
   *    }
   *  ])
   *
   * @param {Array} data - The result of the query execution.
   * @param {Integer} rowCount - The number of affected rows.
   * @param {Array} metadata - Metadata of the returned result set.
   * @param {object} conn - The connection object.
   * @private
   */
  formatResults(data, rowCount, metadata, conn) {
    let result = this.instance;
    if (this.isInsertQuery(data, metadata)) {
      this.handleInsertQuery(data, metadata);
      if (!this.instance) {
        if (this.options.plain) {
          const record = data[0];
          result = record[Object.keys(record)[0]];
        } else {
          result = data;
        }
      }
    }
    if (this.isShowTablesQuery()) {
      result = data;
    } else if (this.isDescribeQuery()) {
      result = {};
      for (const _result of data) {
        if (_result.Default) {
          _result.Default = _result.Default.replace("('", "").replace("')", "").replace(/'/g, "");
        }
        result[_result.Name] = {
          type: _result.Type.toUpperCase(),
          allowNull: _result.IsNull === "Y",
          defaultValue: _result.Default,
          primaryKey: _result.KeySeq > 0,
          autoIncrement: _result.IsIdentity === "Y",
          comment: _result.Comment
        };
      }
    } else if (this.isShowIndexesQuery()) {
      result = this.handleShowIndexesQuery(data);
    } else if (this.isSelectQuery()) {
      result = this.handleSelectQuery(data);
    } else if (this.isUpsertQuery()) {
      result = data;
    } else if (this.isDropSchemaQuery()) {
      result = data[0];
      if (conn) {
        const query = "DROP TABLE ERRORSCHEMA.ERRORTABLE";
        conn.querySync(query);
      }
    } else if (this.isCallQuery()) {
      result = data;
    } else if (this.isBulkUpdateQuery()) {
      result = data.length;
    } else if (this.isBulkDeleteQuery()) {
      result = rowCount;
    } else if (this.isVersionQuery()) {
      result = data[0].VERSION;
    } else if (this.isForeignKeysQuery()) {
      result = data;
    } else if (this.isInsertQuery() || this.isUpdateQuery()) {
      result = [result, rowCount];
    } else if (this.isShowConstraintsQuery()) {
      result = this.handleShowConstraintsQuery(data);
    } else if (this.isRawQuery()) {
      result = [data, metadata];
    } else {
      result = data;
    }
    return result;
  }
  handleShowTablesQuery(results) {
    return results.map((resultSet) => {
      return {
        tableName: resultSet.TABLE_NAME,
        schema: resultSet.TABLE_SCHEMA
      };
    });
  }
  handleShowConstraintsQuery(data) {
    return _.remove(data, (constraint) => {
      return !constraint.constraintName.startsWith("SQL");
    });
  }
  formatError(err, conn, parameters) {
    let match;
    if (!(err && err.message)) {
      err.message = "No error message found.";
    }
    match = err.message.match(/SQL0803N {2}One or more values in the INSERT statement, UPDATE statement, or foreign key update caused by a DELETE statement are not valid because the primary key, unique constraint or unique index identified by "(\d)+" constrains table "(.*)\.(.*)" from having duplicate values for the index key./);
    if (match && match.length > 0) {
      let uniqueIndexName = "";
      let uniqueKey = "";
      const fields = {};
      let message = err.message;
      const query = `SELECT INDNAME FROM SYSCAT.INDEXES  WHERE IID = ${match[1]} AND TABSCHEMA = '${match[2]}' AND TABNAME = '${match[3]}'`;
      if (Boolean(conn) && match.length > 3) {
        uniqueIndexName = conn.querySync(query);
        uniqueIndexName = uniqueIndexName[0].INDNAME;
      }
      if (this.model && Boolean(uniqueIndexName)) {
        uniqueKey = this.model.getIndexes().find((index) => index.unique && index.name === uniqueIndexName);
      }
      if (!uniqueKey && this.options.fields) {
        uniqueKey = this.options.fields[match[1] - 1];
      }
      if (uniqueKey) {
        if (this.options.where && this.options.where[uniqueKey.column] !== void 0) {
          fields[uniqueKey.column] = this.options.where[uniqueKey.column];
        } else if (this.options.instance && this.options.instance.dataValues && this.options.instance.dataValues[uniqueKey.column]) {
          fields[uniqueKey.column] = this.options.instance.dataValues[uniqueKey.column];
        } else if (parameters) {
          fields[uniqueKey.column] = parameters["0"];
        }
      }
      if (uniqueKey && Boolean(uniqueKey.msg)) {
        message = uniqueKey.msg;
      }
      const errors = [];
      _.forOwn(fields, (value, field) => {
        errors.push(new sequelizeErrors.ValidationErrorItem(
          this.getUniqueConstraintErrorMessage(field),
          "unique violation",
          // sequelizeErrors.ValidationErrorItem.Origins.DB,
          field,
          value,
          this.instance,
          "not_unique"
        ));
      });
      return new sequelizeErrors.UniqueConstraintError({ message, errors, cause: err, fields });
    }
    match = err.message.match(/SQL0532N {2}A parent row cannot be deleted because the relationship "(.*)" restricts the deletion/) || err.message.match(/SQL0530N/) || err.message.match(/SQL0531N/);
    if (match && match.length > 0) {
      return new sequelizeErrors.ForeignKeyConstraintError({
        fields: null,
        index: match[1],
        cause: err
      });
    }
    match = err.message.match(/SQL0204N {2}"(.*)" is an undefined name./);
    if (match && match.length > 1) {
      const constraint = match[1];
      let table = err.sql.match(/table "(.+?)"/i);
      table = table ? table[1] : void 0;
      return new sequelizeErrors.UnknownConstraintError({
        message: match[0],
        constraint,
        table,
        cause: err
      });
    }
    return new sequelizeErrors.DatabaseError(err);
  }
  isDropSchemaQuery() {
    let result = false;
    if (this.sql.startsWith("CALL SYSPROC.ADMIN_DROP_SCHEMA")) {
      result = true;
    }
    return result;
  }
  isShowOrDescribeQuery() {
    let result = false;
    result = result || this.sql.toLowerCase().startsWith("select c.column_name as 'name', c.data_type as 'type', c.is_nullable as 'isnull'");
    result = result || this.sql.toLowerCase().startsWith("select tablename = t.name, name = ind.name,");
    result = result || this.sql.toLowerCase().startsWith("exec sys.sp_helpindex @objname");
    return result;
  }
  isShowIndexesQuery() {
    let result = false;
    result = result || this.sql.toLowerCase().startsWith("exec sys.sp_helpindex @objname");
    result = result || this.sql.startsWith('SELECT NAME AS "name", TBNAME AS "tableName", UNIQUERULE AS "keyType", COLNAMES, INDEXTYPE AS "type" FROM SYSIBM.SYSINDEXES');
    return result;
  }
  handleShowIndexesQuery(data) {
    let currItem;
    const result = [];
    for (const item of data) {
      if (!currItem || currItem.name !== item.Key_name) {
        currItem = {
          primary: item.keyType === "P",
          fields: [],
          name: item.name,
          tableName: item.tableName,
          unique: item.keyType === "U",
          type: item.type
        };
        _.forEach(item.COLNAMES.replace(/\+|-/g, (x) => {
          return ` ${x}`;
        }).split(" "), (column) => {
          let columnName = column.trim();
          if (columnName) {
            columnName = columnName.replace(/\+|-/, "");
            currItem.fields.push({
              attribute: columnName,
              length: void 0,
              order: !column.includes("-") ? "ASC" : "DESC",
              collate: void 0
            });
          }
        });
        result.push(currItem);
      }
    }
    return result;
  }
  handleInsertQuery(results, metaData) {
    if (!this.instance) {
      return;
    }
    const modelDefinition = this.model.modelDefinition;
    if (!modelDefinition.autoIncrementAttributeName) {
      return;
    }
    const autoIncrementAttribute = modelDefinition.attributes.get(modelDefinition.autoIncrementAttributeName);
    const id = (results == null ? void 0 : results[0][this.getInsertIdField()]) ?? (metaData == null ? void 0 : metaData[this.getInsertIdField()]) ?? (results == null ? void 0 : results[0][autoIncrementAttribute.columnName]);
    this.instance[autoIncrementAttribute.attributeName] = id;
  }
}
//# sourceMappingURL=query.js.map
